<?php

namespace WPRuby_CAA\Core\Features;

use WPRuby_CAA\Core\Constants;
use WPRuby_CAA\Core\Helpers\Helper;

class Activity_Logger {

	protected static $_instance = null;

	public static function boot()
	{
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}


	public function __construct()
	{
		add_action( 'wp_login', [$this, 'track_login'], 20, 2 );
		add_filter( 'logout_redirect', [$this, 'track_logout'], 10, 3 );
		add_action( 'delete_user', [$this, 'track_delete_user'] );
		add_action( 'edit_user_created_user', [$this, 'track_create_user'] );

		add_action('set_user_role', [$this, 'track_user_role_change'], 10, 3);

		add_action( 'activated_plugin', [$this, 'track_activated_plugin']);
		add_action( 'deactivated_plugin', [$this, 'track_deactivated_plugin'] );
		add_action( 'delete_plugin', [$this, 'track_delete_plugin'] );
		add_action( 'switch_theme', [$this, 'track_switch_theme'] );
		add_action( 'delete_theme', [$this, 'track_theme_deleted'] );
		add_action( 'customize_save', [$this, 'track_customize_save'] );
		add_action( 'export_wp', [$this, 'track_export_wp']  );
		add_action( 'before_delete_post', [$this, 'track_delete_post'], 10, 2  );
		add_action( 'wp_trash_post', [$this, 'track_trash_post']  );
		add_action( 'save_post', [$this, 'track_save_post'], 10, 3 );
		add_action( 'profile_update',  [$this, 'track_profile_update']);

		add_action( 'add_attachment', [$this, 'track_add_attachment'] );
		add_action( 'edit_attachment', [$this, 'track_edit_attachment'] );
		add_action( 'delete_attachment', [$this, 'track_delete_attachment'] );


	}

	/**
	 * @param $action
	 * @param null $user_id
	 * @param null $username
	 * @param null $object_type
	 * @param int $object_id
	 * @param string $object_title
	 */
	public function add_log($action, $user_id = null, $username = null, $object_type = null, $object_id = 0, $object_title = '')
	{
		global $wpdb;

		$user_id = ($user_id === null)? get_current_user_id(): $user_id;
		$username = ($username === null)? wp_get_current_user()->data->user_login : $username;
		$ip_address = Helper::get_user_ip();
		$created_at = current_time( 'mysql' );

		$table_name = Constants::activities_table_name();

		$add_log = $wpdb->insert($table_name, [
			'action' => $action,
			'user_id' => intval($user_id),
			'username' => $username,
			'object_type' => $object_type,
			'object_id' => intval($object_id),
			'object_title' => $object_title,
			'ip_address' => $ip_address,
			'created_at' => $created_at,
		]);


	}

	public function track_login($user_login, $user )
	{
		$this->add_log('user_logged_in', $user->data->ID);
	}

	public function track_logout($redirect_to, $requested_redirect_to, $user)
	{
		$user_id = $user->data->ID;
		$this->add_log('user_logged_out', $user_id);
		return wp_login_url( get_permalink() );
	}

	public function track_delete_user($user_id)
	{
		$this->add_log('user_deleted', null, null, 'user', $user_id);
	}

	public function track_create_user($user_id)
	{
		if ($user_id instanceof \WP_Error) return;
		$user = get_user_by('id', $user_id);
		$this->add_log('user_added', null, null, 'user', $user_id, $user->user_email);
	}

	public function track_user_role_change($user_id, $role, $old_roles)
	{
		$this->add_log('user_role_changed', null, null, 'user', $user_id, $role);
	}

	public function track_activated_plugin($plugin)
	{
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin, true, false );
		$this->add_log('plugin_activated', null, null, 'plugin', 0, $plugin_data['Name']);

	}

	public function track_deactivated_plugin($plugin)
	{
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin, true, false );
		$this->add_log('plugin_deactivated', null, null, 'plugin', 0, $plugin_data['Name']);

	}

	public function track_delete_plugin($plugin)
	{
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin, true, false );
		$this->add_log('plugin_deleted', null, null, 'plugin', 0, $plugin_data['Name']);

	}

	public function track_switch_theme($theme)
	{
		$this->add_log('theme_switched', null, null, 'theme', 0, $theme);
	}

	public function track_customize_save()
	{
		$this->add_log('theme_customizer_saved', null, null, 'theme', 0, 'Theme Customizer');
	}

	public function track_theme_deleted($stylesheet)
	{
		$this->add_log('theme_deleted', null, null, 'theme', 0, $stylesheet);
	}

	public function track_export_wp($args)
	{
		$data    = json_encode($args);
		$this->add_log('content_exported', null, null, 'export', 0, $data);
	}

	/**
	 * @param $post_id
	 * @param \WP_Post $post
	 */
	public function track_delete_post($post_id, $post)
	{
		if ($post->post_type === 'revision') return;

		$this->add_log('post_deleted', null, null, $post->post_type, $post_id, $post->post_title);
	}

	public function track_trash_post($post_id)
	{
		/** @var \WP_Post $post */
		$post = get_post($post_id);
		$this->add_log('post_trashed', null, null, $post->post_type, $post_id, $post->post_title);

	}

	/**
	 * @param int $post_id
	 * @param \WP_Post $post
	 * @param bool $update
	 */
	public function track_save_post($post_id, $post, $update)
	{
		if ($post->post_type === 'revision') return;
		$action = $update? 'post_updated' : 'post_created';
		$this->add_log($action, null, null, $post->post_type, $post_id, $post->post_title);

	}

	public function track_profile_update($user_id)
	{
		$user = get_user_by('id', $user_id);
		$this->add_log('profile_updated', null, null, 'user_profile', $user_id, $user->user_login);
	}

	public function track_add_attachment($attachment_id)
	{
		$attachment_title = get_the_title($attachment_id);
		$this->add_log('attachment_added', null, null, 'attachment', $attachment_id, $attachment_title);

	}

	public function track_edit_attachment($attachment_id)
	{
		$attachment_title = get_the_title($attachment_id);
		$this->add_log('attachment_updated', null, null, 'attachment', $attachment_id, $attachment_title);

	}

	public function track_delete_attachment($attachment_id)
	{
		$attachment_title = get_the_title($attachment_id);
		$this->add_log('attachment_deleted', null, null, 'attachment', $attachment_id, $attachment_title);

	}

}
